#
# spec file for package cpupower
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
# Author: Thomas Renninger <trenn@suse.de>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           cpupower
# Use this as version when things are in mainline kernel
%define version %(rpm -q --qf '%VERSION' kernel-source)
Version:        4.15
Release:        93.1
%define tsversion      17.04.12
%define pbversion      17.05.11
Summary:        Tools to determine and set CPU Power related Settings
License:        GPL-2.0
Group:          System/Base
Url:            https://git.kernel.org/cgit/linux/kernel/git/rafael/linux-pm.git
Source:         %name-%version.tar.bz2
Source1:        turbostat-%tsversion.tar.bz2
Source2:        cpupower_export_tarball_from_git.sh
Source3:        x86_energy_perf_policy-%{pbversion}.tar.bz2

Patch1:         cpupower_rapl.patch
Patch2:         rapl_monitor.patch
Patch3:         cpupower_exclude_kernel_Makefile.patch

Patch20:        turbostat_fix_man_perm.patch
Patch22:        turbostat_makefile_fix_asm_header.patch
# Fixes bsc#1048546:
Patch23:        turbostat_decode_MSR_IA32_MISC_ENABLE_only_on_Intel.patch

Patch30:        x86_perf_makefile_fix_asm_header.patch
Patch31:        x86_perf_fix_man_permissions.patch

BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  gettext-tools
BuildRequires:  pciutils
BuildRequires:  pciutils-devel
Obsoletes:      cpufrequtils
Provides:       cpufrequtils = %version

%description
This tool is to make access to the Linux kernel's processor power subsystems
like CPU frequency switching (cpufreq) or CPU sleep states (cpuidle) for users
and userspace tools easier.

%package -n libcpupower0
Summary:        Obsolete processor frequency related C-library
Group:          System/Libraries

%description -n libcpupower0
Contains libcpupower and soon possibly other CPU power related
C libraries.

%package devel
Summary:        Include files and C-libraries
Group:          Development/Languages/C and C++
Conflicts:      cpufrequtils-devel
Requires:       libcpupower0 = %version

%description devel
Include files and C-libraries for C/C++ development

%package bench
Summary:        CPU frequency micro benchmark
Group:          System/Benchmark
Obsoletes:      cpufrequtils-bench < %version
Provides:       cpufrequtils-bench = %version

%description bench
Helps to test the condition of a given kernel cpufreq
governor (e.g. ondemand, userspace, conservative) and the cpufreq HW driver
(e.g. powernow-k8, acpi-cpufreq, ...).
For that purpose, it compares the performance governor to a configured
powersave module.

%prep
%setup -D -b 1 -b 3
%patch1 -p1
%patch2 -p1
%patch3 -p1

cd ../turbostat-%tsversion
%patch20 -p1
%patch22 -p1
%patch23 -p1

cd ../x86_energy_perf_policy-%{pbversion}
%patch30 -p1
%patch31 -p1

%build
# This package failed when testing with -Wl,-as-needed being default.
# So we disable it here, if you want to retest, just delete this comment and the line below.
export SUSE_ASNEEDED=0
CONF="PACKAGE_BUGREPORT=http://bugs.opensuse.org mandir=%_mandir libdir=%_libdir CPUFRQ_BENCH=true VERSION=%version"
export CFLAGS="%optflags -I ."
make $CONF %{?_smp_mflags}

%ifarch ix86 x86_64
cd ../turbostat-%tsversion
export CFLAGS="%optflags -I ."
make %{?_smp_mflags}
cd ../x86_energy_perf_policy-%{pbversion}
make %{?_smp_mflags}
%endif

%install
export SUSE_ASNEEDED=0
CONF="PACKAGE_BUGREPORT=http://bugs.opensuse.org mandir=%_mandir libdir=%_libdir CPUFRQ_BENCH=true DESTDIR=%buildroot sbindir=%_sbindir docdir=%_docdir/%name confdir=%_sysconfdir VERSION=%version"
%make_install $CONF
# copy to examples doc dir to avoid complains from the build
# system about an executable in the doc dir.
mkdir -p %buildroot/%_docdir/%name/examples
mv %buildroot//%_docdir/%name/cpufreq-bench_script.sh %buildroot/%_docdir/%name/examples
%find_lang %name

%ifarch ix86 x86_64
cd ../turbostat-%tsversion
%make_install -e
cd ../x86_energy_perf_policy-%{pbversion}
%make_install
%endif

%post -n libcpupower0 -p /sbin/ldconfig

%postun -n libcpupower0 -p /sbin/ldconfig

%files -f %name.lang
%defattr (-,root,root)
%_mandir/man1/cpupower*
%_bindir/cpupower
%ifarch ix86 x86_64
%_mandir/man8/turbostat*
%_bindir/turbostat
%_mandir/man8/x86_energy_perf_policy*
%_bindir/x86_energy_perf_policy
%endif

%files bench
%defattr (-,root,root)
%config %_sysconfdir/cpufreq-bench.conf
%_sbindir/cpufreq-bench
%_bindir/cpufreq-bench_plot.sh
%dir %_docdir/%name
%dir %_docdir/%name/examples
%_docdir/%name/examples/cpufreq-bench_script.sh
%_docdir/%name/README-BENCH

%files -n libcpupower0
%defattr(-,root,root)
%_libdir/libcpupower*.so.*

%files devel
%defattr(-,root,root)
%_includedir/cpufreq.h
%_includedir/cpuidle.h
/usr/include/powercap.h
%_libdir/libcpu*.so

%changelog
* Thu Jan 11 2018 trenn@suse.de
- Update cpupower to latest kernel version (version name 4.15, but
  checked out against latest kernel tag 4.15-rc7. There will not
  be important changes any more, maybe a fix).
- cpu online/offline fixes
- This is the first cpupower package (with updated version) which
  includes x86_energy_perf_policy binary.
  This is important for later package dependencies, namely tuned.
  A cpupower_exclude_kernel_Makefile.patch
* Tue Jan  9 2018 trenn@suse.de
- Add x86_energy_perf_bias tool
  This is a tool which is, same as turbostat, located in kernel sources here:
  tools/power/x86/
  A x86_energy_perf_policy-17.05.11.tar.bz2
  A x86_perf_fix_man_permissions.patch
  A x86_perf_makefile_fix_asm_header.patch
* Fri Oct 20 2017 trenn@suse.de
  fate#321274
- Provide rapl domain info (cpupower powercap-info cmd)
  * Add: cpupower_rapl.patch
- Provide rapl power monitoring
  * Add: rapl_monitor.patch
* Fri Oct  6 2017 josef.moellers@suse.com
- Added missing references.
  [bsc#1041332,bsc#996052,bsc#976983]
* Thu Oct  5 2017 jengelh@inai.de
- Rectify RPM groups. Replace old $RPM_ shell variables by macros.
  Apply consistent styling to macros.
* Thu Oct  5 2017 josef.moellers@suse.com
- Enter decode_misc_enable_msr() only if genuine_intel.
  [bsc#1048546,
  turbostat_decode_MSR_IA32_MISC_ENABLE_only_on_Intel.patch]
* Wed May  3 2017 trenn@suse.de
- Update to latest mainline sources
- turbostat changed versioning scheme (we now have version 17.04.12)
* Wed Mar 22 2017 trenn@suse.de
- Update cpupower and turbostat to latest kernel v4.10 sources
  D library_cleanup.patch
- Change .c file header file modification, modify in Makefile instead:
  D turbostat_set_asm_header_fixed.patch
  A turbostat_makefile_fix_asm_header.patch
- fate #320908: KBL: Turbostat for KBL H/S & U/Y
- fate #321191: [KNL] turbostat reports to report corect MSR_TURBO_RATIO_LIMIT
  information
- fate #321193: [KNL] turbostat should report correct cpu clock values for
  Knights Landing
- fate #321285: turbostat support for Skylake-SP server
- fate #321286: turbostat support for Harrisonville (Denverton SoC)
- fate #321925: [HPC, KNM, kernel] turbostat support
* Thu Apr 28 2016 trenn@suse.de
- Update turbostat to version 4.12
- Update cpupower to latest sources
- Let turbostat only build against a local msr-index.h
  Also add the msr-index.h export to the tarball from git repo script
  * Delete make_header_file_passable_from_outside.patch
* Wed Apr 20 2016 trenn@suse.de
- Add cpuidle functions to public libcpupower
  * Add library_cleanup.patch
* Tue Jan 26 2016 trenn@suse.de
- Move from kernel version to turbostat internal version for turbostat
- Add latest turbostat sources (latest git commit: 30f05309bde492)
- Adds Skylake, Broadwell and Knights Landing support for turbostat
  (fate#319798, fate#319183, fate#319516)
* Wed Jan  6 2016 jdelvare@suse.com
- Drop dead link.
- Fix Obsoletes statement, cpufrequtils package versions were
  greater than cpupower package versions so we must obsolete all
  versions of cpufrequtils.
* Wed Jan 28 2015 trenn@suse.de
- Make sources also build against 13.1 and older by providing
  asm-index.h which is included in linux-glibc-devel in newer
  distributions
  * Add turbostat_set_asm_header_fixed.patch
* Wed Jan 28 2015 trenn@suse.de
- Update cpupower and turbostat to latest git kernel HEAD version
  this currently is 3.19-rc6, but it is expected that there are no
  changes anymore and the version gets named 3.19 already.
  The patches are already included mainline and get removed from osc:
    cpupower_disable_by_latency.patch
    add_idle_manpages_in_Makefile.patch
    cpupower-always_try_to_load_msr_driver_as_root.patch
    cpupower-return_zero_on_success.patch
    cpupower-remove_mc_scheduler_bits.patch
* Wed Aug 13 2014 trenn@suse.de
- Fix cpupower_export_tarball_from_git.sh script and make sure msr-index.h
  is included in the turbostat tarball.
* Tue Aug 12 2014 trenn@suse.de
- Update turbostat to latest 3.17 kernel version (in fact it is 3.16-rc7)
  This fixes bnc#891523
* Tue Apr  1 2014 trenn@suse.de
- Do return 0 if cpupower info -b is called instead of the perf bias value
  which is already printed to stdout.
  Add cpupower-return_zero_on_success.patch
- Remove mc and smt scheduler knobs. These do not exist anymore in latest
  kernels for quite a while now.
  Add cpupower-remove_mc_scheduler_bits.patch
* Mon Mar 31 2014 trenn@suse.de
- If root, try to load msr driver on x86 if /dev/cpu/0/msr is not avail
  Add cpupower-always_try_to_load_msr_driver_as_root.patch
* Fri Mar 28 2014 trenn@suse.de
- Update to latest turbostat version (git 2d6e532279ddf30188d5fbd)
  bnc#866862, bnc#869071, bnc#869809
- Adjust MSR include path
  add make_header_file_passable_from_outside.patch
* Fri Mar 28 2014 trenn@suse.de
- Fix Makefile: Install latest cpupower-idle-* manpages
  Add add_idle_manpages_in_Makefile.patch
* Fri Mar 14 2014 trenn@suse.de
- Introduce idle state disable-by-latency and enable-all
  Part of fate#316611
  Add cpupower_disable_by_latency.patch
* Fri Feb 14 2014 trenn@suse.de
- turbostat is an x86 (in fact an Intel) only tool
  exclude it for other archs to fix building of the package there again
* Fri Jan 31 2014 trenn@suse.de
- Update cpupower to latest kernel version (3.13-rc7, rc7 missed out,
  there won't be any changes in cpupower anymore and obs does not like
  - rc7 in version tag.
- All patches are now included mainline, therefore:
  Drop cpupower_make_get_idlestate_count_unsigned.patch
  Drop cpupower_disable_cstates.patch
  Drop introduce_cpupower_idle_set.patch
  Drop cpupower-haswell_support.patch
  Drop cpupower_hsw_special.patch
- Add turbostat utility (fate#315666):
  Add turbostat-3.13.tar.bz2
  Add turbostat_fix_man_perm.patch
* Mon Jul  1 2013 trenn@suse.de
- Add disable C-state capability and support for latest Haswell CPUs
* Fri May 31 2013 varkoly@suse.com
- Fix spec to avoid conflict with cpufrequtils-devel
* Wed Feb 13 2013 trenn@suse.de
- Add Haswell support
* Wed Jan 23 2013 trenn@suse.de
- Update to kernel version 3.7
  * IvyBridge support integrated
* Tue Nov 13 2012 trenn@suse.de
- Rename SandyBridge monitor to IvyBridge if CPU is an IvyBridge
* Tue Nov 13 2012 trenn@suse.de
- Add ivy bridge processor (EP and EX) support
* Tue May  8 2012 trenn@suse.de
- Update to latest 3.4.rc6 kernel based version
* Wed Dec 21 2011 aj@suse.de
- Add cpupower_export_tarball_from_git.sh as source file.
* Tue Dec 20 2011 trenn@suse.de
- Add idle-info manpage
- Fix two bugs: memleak in idle-info and wrong idle state count
* Thu Dec 15 2011 trenn@suse.de
- Update to 3.2-rc5 version (not much changed, includes the
  use-manpage-on-help patch -> removed it)
- Change licence from GPL-2.0+ to GPL-2.0 to fit the kernel
  licence where sources are coming from
- Provide a tiny script to quickly export and tar sources from
  the git repo
* Thu Dec 15 2011 coolo@suse.com
- fix license to be in spdx.org format
* Sat Sep 17 2011 jengelh@medozas.de
- Remove redundant tags/sections from specfile
- Use %%_smp_mflags for parallel build
* Thu Aug 25 2011 trenn@suse.de
- Use manpages for help commands
* Mon May  2 2011 trenn@suse.de
- cpupower-bench obsoletes cpufrequtils-bench
* Thu Apr 21 2011 trenn@suse.de
- Initial commit of tools/power/cpupower directory from
  linux-next kernel repository
